@@ -147,12 +147,32 @@ recover_data = function(object, ...) {
 #'   and the default value \code{"pi"} (the only numeric constant in base R)
 #'   is provided in case the model involves it. An example involving splines
 #'   may be found at \url{https://github.com/rvlenth/emmeans/issues/180}.
+#' @param frame Optional \code{data.frame}. Many model objects contain the 
+#'   model frame used when fitting the model. In cases where there are no 
+#'   predictor transformations, this model frame has all the original predictor
+#'   values and so is usable for recovering the data. Thus, if \code{frame} is
+#'   non-missing and \code{data} is \code{NULL}, a check is made on \code{trms}
+#'   and if there are no function calls, we use \code{data = frame}. This
+#'   can be helpful because it provides a modicum of security against the
+#'   possibility that the original data used when fitting the model has been
+#'   altered or removed.
+#' @param addl.vars Character value or vector specifying additional predictors
+#'   to include in the reference grid. These must be names of variables that
+#'   exist, or you will get an error. 
+#'   This may be useful if you need to do
+#'   additional computations later on that depend on these variables; e.g., 
+#'   bias adjustments for random slopes of variables not among the fixed predictors.
 #' 
 #' @method recover_data call
 #' @export
-recover_data.call = function(object, trms, na.action, data = NULL, params = "pi", ...) {
+recover_data.call = function(object, trms, na.action, data = NULL, 
+                             params = "pi", frame, addl.vars, ...) {
     fcall = object # because I'm easily confused
     vars = setdiff(.all.vars(trms), params)
+    if(!missing(addl.vars))
+        vars = union(vars, addl.vars)
+    if (!missing(frame) && is.null(data) && !.has.fcns(trms))
+        data = frame
     tbl = data
     if (length(vars) == 0 || vars[1] == "1") {
         tbl = data.frame(c(1,1))
@@ -218,6 +238,10 @@ recover_data.call = function(object, trms, na.action, data = NULL, params = "pi"
     attr(tbl, "call") = object # the original call
     attr(tbl, "terms") = trms
     attr(tbl, "predictors") = setdiff(.all.vars(delete.response(trms)), params)
+    if(!missing(addl.vars)) {
+        attr(tbl, "predictors") = union(attr(tbl, "predictors"), addl.vars)
+        vars = setdiff(vars, addl.vars)
+    }
     attr(tbl, "responses") = setdiff(vars, union(attr(tbl, "predictors"), params))
     tbl
 }
