@@ -1,5 +1,5 @@
 ##############################################################################
-#    Copyright (c) 2012-2016 Russell V. Lenth                                #
+#    Copyright (c) 2012-2022 Russell V. Lenth                                #
 #                                                                            #
 #    This file is part of the emmeans package for R (*emmeans*)              #
 #                                                                            #
@@ -146,8 +146,9 @@ emm_basis.clm = function (object, trms, xlev, grid,
             nbasis = estimability::nonest.basis(NOMX)
             # replicate and reverse the sign of the NOM parts
             nomcols = seq_len(ncol(NOM))
+            nomreps = length(object$alpha) / ncol(NOM)
             nbasis = apply(nbasis, 2, function(x)
-                c(rep(-x[nomcols], each = nrow(NOM)), x[-nomcols]))
+                c(rep(-x[nomcols], each = nomreps), x[-nomcols]))
         }
         if (!is.null(mm$S)) {
             if (any(is.na(object$zeta))) {
@@ -385,3 +386,18 @@ emm_basis.clmm = function (object, trms, xlev, grid, ...) {
     result$V = result$V[keep,keep]
     result
 }
+
+
+### Support for 'svyolr' objects (survey package)
+### It appears all we have to do is pretend it is class polr
+### and get the data via model.frame
+recover_data.svyolr = function(object, data = NULL, ...) {
+    if (is.null(data))
+        data = get("model.frame.svyolr", asNamespace("survey"))(object)
+    class(object) = "polr"
+    recover_data(object, data = data, ...)
+}
+emm_basis.svyolr = function(object, ...) {
+    class(object) = "polr"
+    emm_basis(object, ...)
+}
